dojo.provide("zero.grid.DataGrid");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("dojox.grid.Grid");
dojo.require("dojox.grid._data.model");
dojo.require("dojox.grid._data.dijitEditors");
dojo.require("zero.resource.DataStore");

dojo.declare('zero.grid.DataGrid', 
	 [dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("zero.grid","DataGrid/DataGrid.html"),
	widgetsInTemplate: true,

	editorMap: {"string": dojox.grid.editors.Input,// Map of datatypes to property editors
				"enum" : dojox.grid.editors.Select,// string with choices[] specified
				"datetime": dojox.grid.editors.DateTextBox,
				"bool": dojox.grid.editors.Bool,
				"auto": undefined},
	defaultLayout:[{type: 'dojox.GridRowView', width: '20px'},
					{
						defaultCell: { width: 8, styles: 'text-align: right;'  },
						rows: [[{ name: 'Field1', styles: 'text-align: center;'},
							 	{ name: 'Field2', styles: 'text-align: center;'},
							 	{ name: 'Field3', styles: 'text-align: center;'}]]}],
	defaultModel: null,

	grid: null,
	contextRoot: "",
	resourceCollection: "",
	query: {},
	disabled: false,
	readonly: false,
	rowsPerPage: 20,
	store: null,
	visibleFields: "", // String array of field names to display (comma delim)

	constructor: function(){
	   console.debug("exiting constructor");
		this.defaultModel = new dojox.grid.data.Table(null, [["?","?","?"]]);		
	},
	
	postMixInProperties: function(){
	   console.debug("exiting postMixInProperties");
		this.inherited('postMixInProperties', arguments);
		//FIXME: Localize button text here...
/*		this.messages = dojo.i18n.getLocalization("dijit", "common", this.lang);
		dojo.forEach(["buttonSave", "buttonCancel"], function(prop){
			if(!this[prop]){ this[prop] = this.messages[prop]; }
		}, this);
*/
		this.defaultModel = new dojox.grid.data.Table(null, [["?","?","?"]]);		
	},

	_isReady: function(){
		var rdy = true;
		if( !this.store && (this.resourceCollection === null || this.resourceCollection.length == 0)){
			console.debug("DataGrid.postCreate: Missing required attribute, resourceCollection (ex. 'employees')");
			rdy = false;
		}
		return rdy;
	},

	postCreate: function(){
		this.grid = new dojox.Grid({},this.gridNode);
		if(this._isReady()){
			if(!this.store){// create a default ZRM store if external store is not specified
				this.store = new zero.resource.DataStore({contextRoot:this.contextRoot, resourceCollection:this.resourceCollection});
			} else {
				if (this.resourceCollection.length > 0 || this.contextRoot.length > 0) {
					console.debug("Ignoring resourceCollection or contextRoot attribute set in DataGrid in favor of those specified in store:"
						+ this.store);
				}
			}
			
			if(!this.grid.rowsPerPage){this.grid.rowsPerPage = this.rowsPerPage;}
			this.grid._setModel(new dojox.grid.data.DojoData(null,null,{ 
					query: this.query, 
					store:this.store, 
					rowsPerPage: this.rowsPerPage }));
		}else{
			this.readonly = true;
			this.grid.setModel(this.defaultModel);
		}
		this.grid.setStructure(this.buildLayout());
		this.grid.singleClickEdit=true;
		this._buildToolbar();
		this.reload();
		this.inherited("postCreate", arguments);
	},
	
	startup: function(){
		this.grid.render();
	},
	
	_buildToolbar: function(){
		this._handlers = (this._handlers || []).concat([
		    dojo.connect(this.createButton,"onClick",this,"addRow"),
			dojo.connect(this.deleteButton,"onClick",this,"deleteRows"),
			dojo.connect(this.refreshButton,"onClick",this,"reload"),
			dojo.connect(this.saveButton,"onClick",this,"save"),
		]);
		if (this.readonly){ this.toolbar.domNode.style.display = "none"; }
	},

	_cleanupHandlers: function(){
		dojo.forEach(this._handlers, dojo.disconnect);
		delete this._handlers;
	},
	
	buildLayout: function(){
		// Builds a layout structure from datastore's metadata (using experimental ZRM metadata)
		var layout = 
			[{
			cells: [[
				// inserting cells here
			]]}];
		var md = this.store? 
			this.store.getMetadata(): 
			null;
		var vf = this.visibleFields? 
			this.visibleFields.split(","): 
			null;
			

		if(vf && vf.length > 0){
			if (!md) {
				for (var i = 0; i < vf.length; i++){
					var field = {type:"string", label:vf[i]};
					layout[0].cells[0].push(this._buildCell(vf[i], field));
				}
			} else {
				for (var i = 0; i < vf.length; i++){
					layout[0].cells[0].push(this._buildCell(vf[i], md.fields[vf[i]]));
				}
			}
		}else if(md){
			for (var fname in md.fields){
				if (fname){
					layout[0].cells[0].push(this._buildCell(fname, md.fields[fname]));
				}
			}
		}else{
			layout = this.defaultLayout;
		}
		return layout;
	},
	
	_buildCell: function(id, field){
		var editor;
		if (field.type){
			if (!this.readonly && (field.type !== "auto") && (!field.readonly)){
				if (field.options && field.options.length > 0){
					editor = this.editorMap["enum"];
				}else{			
					editor = this.editorMap[field.type];
				}
			}
		}
		return {name: (field.label ? field.label : id),
				field: id,
				styles: (field.styles ? field.styles : 'text-align: center;height:19px;'),
				editor: editor,
				options: field.options };
	},

	setVisibleFields: function(fieldNames){
		this.visibleFields = fieldNames;
		this.grid.setStructure(this.buildLayout());
	},
	
	render: function(){
		this.grid.render();
	},
	
	destroy: function(){
		this.grid.setModel(null);
		this.store=null;
		this._cleanupHandlers();
		this.inherited("destroy", arguments);
	},
	
	save: function(){
		if (!this.readonly){
			this.store.save({
				onComplete: dojo.hitch(this,this.onSaved),//Noop 
				onError: dojo.hitch(this,this.onError)});
		}
	},

	revert: function(){
		if (!this.readonly){
			this.store.revert();
		}
	},
	
	reload: function(){
		if (this.store && this.grid.model && this.grid.model.refresh){
			this.grid.model.refresh();
		}
	},
	
	addRow:function(){
		if (!this.readonly){
			this.grid.addRow(this.store.createDefaultItem());
		}
	},
	
	deleteRows:function(){
		if (!this.readonly){
			this.grid.removeSelectedRows();
		}
	},

	onSaved: function(){
		// onComplete handling for save()
		if (this.grid.model){
			this.grid.model.refresh();
		}
	},

	onError: function(errorData){
		console.log('Error: ' + errorData);
	}
});
